function p = isAcyclic(adj_mat, directed)
% isAacyclic True iff  this graph has no (directed) cycles
% function p = isAcyclic(adj_mat, directed)

adj_mat = double(adj_mat);

% e.g., G =
% 1 -> 3
%      |
%      v
% 2 <- 4   
% In this case, 1->2 in the transitive closure, but 1 cannot get to itself.
% If G was undirected, 1 could get to itself, but this graph is not cyclic.
% So we cannot use the closure test in the undirected case.

if directed
  % compute transitive closure - faster than dfs
  G = adj_mat;
  % expm(G) = I + G + G^2 / 2! + G^3 / 3! + ...
  M = expm(double(full(G))) - eye(length(G));
  R = (M>0);
  p = ~any(diag(R)==1);
else
  [d, f, pred, cycle] = depthFirstSearch(adj_mat, [], directed);
  p = ~cycle;    
end
